/**
 * Client requests for the memaccess skin
 *
 * This include file contains some GCCism. If you want to use this
 * with anything other than the GCC C compiler, link the file ma_wrapper.c
 * in the tests/ directory, compiled with GCC, to your project.
 *
 * 2003, Josef Weidendorfer
 * Distributed under LGPL.
 */

#ifndef __MEMACCESS_H
#define __MEMACCESS_H

#define __VALGRIND_SOMESKIN_H
#include "valgrind.h"


/* Please use these types for the callbacks.
 * The macros below unfortunately can't check types.
 */
typedef void (*MAInstrReadProto)(int, void*, int);
typedef void (*MADataReadProto)(int, void*, void*, int);
typedef void (*MADataWriteProto)(int, void*, void*, int);

/*
 * Valgrind core functions to be used in callbacks instead
 * of the libc variants (see MEMACCESS_DEFINE_CALLBACKS macro below).
 *
 * MAFuncPrintf always writes to stderr on default. Can be redirected
 * with --logfile-fd=N like other Valgrind output.
 */
typedef unsigned int (*MAFuncPrintf)(const char*, ...);
typedef void*        (*MAFuncMalloc)(int);
typedef void         (*MAFuncFree)(void*);
typedef void         (*MAFuncExit)(int);
typedef void         (*MAFuncPanic)(signed char*);

typedef struct _maFuncTable
{
    MAFuncPrintf printf;
    MAFuncMalloc malloc;
    MAFuncFree   free;
    MAFuncExit   exit;
    MAFuncPanic  panic;
} MAFuncTable;


/* Following enum is used internally in the macros defined below */    

typedef
   enum {
      MA_USERREQ__DEFINE_CALLBACKS = VG_USERREQ_SKIN_BASE('M','A'),
      MA_USERREQ__START,
      MA_USERREQ__STOP
   } Vg_CalltreeClientRequest;


/* Notify the Memaccess skin about callbacks to be done
 * on memory access events.
 * Returned is a list of pointers to replacement functions for libc
 * routines to be called in the callbacks. This is needed as the
 * callbacks run in the tool environment parallel to the supervised
 * application and aren't allowed to call functions which simultaniously
 * could be called by the application itself.
 */
#define MEMACCESS_DEFINE_CALLBACKS(_qzz_arg0, _qzz_arg1, _qzz_arg2) \
   (__extension__({MAFuncTable* _qzz_res;                           \
     VALGRIND_MAGIC_SEQUENCE(_qzz_res, 0,                           \
			     MA_USERREQ__DEFINE_CALLBACKS,          \
			     _qzz_arg0, _qzz_arg1, _qzz_arg2, 0);   \
     _qzz_res;	 						    \
   }))


/* Start notifying the application */
#define MEMACCESS_START()						\
   do {									\
     unsigned int _qzz_res;						\
     VALGRIND_MAGIC_SEQUENCE(_qzz_res, 0, MA_USERREQ__START,	        \
			     0, 0, 0, 0);				\
     (void)0;								\
   } while(0)


/* Stop notifying the application */
#define MEMACCESS_STOP()						\
   do {									\
     unsigned int _qzz_res;						\
     VALGRIND_MAGIC_SEQUENCE(_qzz_res, 0, MA_USERREQ__STOP,	        \
			     0, 0, 0, 0);				\
     (void)0;								\
   } while(0)

#endif /* __MEMACCESS_H */
