/**
 * Definition of wrapper functions using the
 * macros with client requests from memaccess.h.
 * Futher, replacement functions of Valgrind core for LIBC functions
 * are supplied.
 *
 * As memaccess.h contains some GCCism, this file defines functions to
 * be called from your application if you don't use GCC.
 * Still, this file has to be compiled with GCC.
 *
 * 2003, Josef Weidendorfer
 * Distributed under LGPL.
 */

#include "memaccess.h"

MAFuncTable* memaccess_ft = 0;

void memaccess_define_callbacks(MAInstrReadProto ir,
				MADataReadProto dr,
				MADataWriteProto dw)
{
  memaccess_ft = MEMACCESS_DEFINE_CALLBACKS(ir,dr,dw);
}


void memaccess_start()
{

  MEMACCESS_START();

}

void memaccess_stop()
{
  MEMACCESS_STOP();
}

/* To call the printf function with a variable count of arguments,
 * directly use the function pointer like
 *
 * memaccess_ft->printf(<format>,<arg1>,<arg2>,...);
 */
unsigned int memaccess_printf(const char* str)
{
    if (!memaccess_ft) return 0;

    return memaccess_ft->printf(str);
}

unsigned int memaccess_printf1(const char* str, int arg1)
{
    if (!memaccess_ft) return 0;

    return memaccess_ft->printf(str,arg1);
}

unsigned int memaccess_printf2(const char* str, int arg1, int arg2)
{
    if (!memaccess_ft) return 0;

    return memaccess_ft->printf(str,arg1,arg2);
}

unsigned int memaccess_printf3(const char* str, int arg1, int arg2, int arg3)
{
    if (!memaccess_ft) return 0;

    return memaccess_ft->printf(str,arg1,arg2,arg3);
}

void* memaccess_malloc(int size)
{
    if (!memaccess_ft) return 0;
    
    return memaccess_ft->malloc(size);
}

void memaccess_free(void* buf)
{
    if (!memaccess_ft) return;

    memaccess_ft->free(buf);
}

void memaccess_exit(int ret)
{
    if (!memaccess_ft) return;

    memaccess_ft->exit(ret);
}

void memaccess_panic(signed char* str)
{
    if (!memaccess_ft) return;

    memaccess_ft->panic(str);
}

