/*
 * Test for the MemAccess Skin using macros from memaccess.h
 *
 */

#include <stdio.h>
#include "memaccess.h"

/* Ring buffers for memaccess trace */
#define TYPE_INSTR 1
#define TYPE_READ  2
#define TYPE_WRITE 3

#define TRACEBUFSIZE 2000

int tracecount = 0;
int* type_array = 0;
int* tid_array;
void** addr_array;
int* size_array;

/* This pointer is set with MEMACCESS_DEFINE_CALLBACKS in main.
 * It defines pointers to functions in Valgrind core
 * for code executed in callbacks called from the memaccess skin.
 */
MAFuncTable* ma_ft = 0;

void init_tracebuffer()
{
    type_array = ma_ft->malloc(TRACEBUFSIZE * sizeof(int));
    tid_array  = ma_ft->malloc(TRACEBUFSIZE * sizeof(int));
    addr_array = ma_ft->malloc(TRACEBUFSIZE * sizeof(void*));
    size_array = ma_ft->malloc(TRACEBUFSIZE * sizeof(int));
}

void ir(int tid, void* iaddr, int isize)
{
    if (!type_array) init_tracebuffer();

    type_array[tracecount] = TYPE_INSTR;
    tid_array[tracecount]  = tid;
    addr_array[tracecount] = iaddr;
    size_array[tracecount] = isize;
    tracecount++;

    if (tracecount == TRACEBUFSIZE)
	ma_ft->panic("Overflow in trace event buffer");

    ma_ft->printf("Ir for instruction %x\n", iaddr);
}

void dr(int tid, void* iaddr, void* daddr, int dsize)
{
    if (!type_array) init_tracebuffer();

    type_array[tracecount] = TYPE_READ;
    tid_array[tracecount]  = tid;
    addr_array[tracecount] = daddr;
    size_array[tracecount] = dsize;
    tracecount++;

    if (tracecount == TRACEBUFSIZE)
	ma_ft->panic("Overflow in trace event buffer");

    ma_ft->printf("Dr at instruction %x for data at %x\n", iaddr, daddr);
}

void dw(int tid, void* iaddr, void* daddr, int dsize)
{
    if (!type_array) init_tracebuffer();

    type_array[tracecount] = TYPE_WRITE;
    tid_array[tracecount]  = tid;
    addr_array[tracecount] = daddr;
    size_array[tracecount] = dsize;
    tracecount++;

    if (tracecount == TRACEBUFSIZE)
	ma_ft->panic("Overflow in trace event buffer");

    ma_ft->printf("Dw at instruction %x for data at %x\n", iaddr, daddr);
}


int main()
{
    int i, a[100];

    ma_ft = MEMACCESS_DEFINE_CALLBACKS(ir,dr,dw);

    if (0) for(i=0;i<100;i++) a[i] = i;

    MEMACCESS_START();

    /* This loop will be traced */
    for(i=0;i<100;i++) a[i] += i;

    MEMACCESS_STOP();

    /* show trace */
    printf("Trace Events: %d\n", tracecount);
    for(i=0;i<tracecount;i++) {
	switch(type_array[i]) {
	case TYPE_INSTR:
	    printf("[%5d] TID %2d: I %p (size %d)\n", i, 
		   tid_array[i], addr_array[i], size_array[i]);
	    break;
	case TYPE_READ:
	    printf("[%5d] TID %2d: R %p (size %d)\n", i, 
		   tid_array[i], addr_array[i], size_array[i]);
	    break;
	case TYPE_WRITE:
	    printf("[%5d] TID %2d: W %p (size %d)\n", i, 
		   tid_array[i], addr_array[i], size_array[i]);
	    break;
	default:
	    printf("[%5d] ERROR: Unknown type\n", i);
	}
    }

    return 1;
}
